@@ -1,57 +1,29 @@
 ##### Distributed Logistic Multinomial Regression  ######
 
 ## define class
-setClass("dmrcoef",
-  representation(lambda="numeric"), 
-  contains="dgCMatrix")
-
-## inner loop function(s)
-linapprox <- function(xj, argl){
-  if(!is.null(argl$zeta))
-    zeta <- argl$zeta 
-  else 
-    zeta <- 0.05
-  cat(sprintf("linear approx with zeta = %g\n",zeta))
-
-  if(zeta>0){
-    argl$y <- log(xj[,1]+zeta) - argl$fix
-    argl$obsweight <- xj[,1]+zeta
-  } 
-  else{
-    argl$y <- log(xj@x) - argl$fix[xj@i+1]
-    argl$obsweight <- xj@x
-    argl$x <- argl$x[xj@i+1,,drop=FALSE]
-  }
-
-  argl$fix <- NULL
-
-  fit <- do.call(gamlr,argl)
-  if(length(fit$lambda)<argl$nlambda) 
-    print(colnames(xj))
-
-  return(fit)
-}
+setClass("dmrcoef", contains="dgCMatrix")
 
+## inner loop function
 onerun <- function(xj, argl){
-  if(argl$family=="gaussian")
-    return(linapprox(xj,argl))
   argl$y <- xj
-  fit <- do.call(gamlr,argl)
-  ## print works only if you've specified an outfile in makeCluster
-  if(length(fit$lambda)<argl$nlambda) print(colnames(xj))
-  return(fit)
+  if(argl$cv) fit <- do.call(cv.gamlr,argl)
+  else fit <- do.call(gamlr,argl)
+   return(fit)
 }
 
 ## main function
-dmr <- function(cl, covars, counts, mu=NULL, bins=NULL, verb=0, ...)
+dmr <- function(cl, covars, counts, mu=NULL, bins=NULL, verb=0, cv=FALSE, ...)
 {
+  if(!is.null(cl)){
+    if(!inherits(cl,"cluster")) stop("first argument `cl' must be NULL or a socket cluster.")
+  }
   #build the default argument list
   argl <- list(...)
-  if(is.null(argl$family))
-    argl$family="poisson"
+  argl$family <- "poisson"
   if(is.null(argl$nlambda))
     argl$nlambda <- formals(gamlr)$nlambda
   argl$verb <- max(verb-1,0)
+  argl$cv <- cv
 
   ## collapse and clean
   chk <- collapse(covars, counts, mu, bins)
@@ -105,53 +77,22 @@ dmr <- function(cl, covars, counts, mu=NULL, bins=NULL, verb=0, ...)
   return(mods)
 }
 
-coef.dmr <- function(object, select=NULL, k=2, ...){
-  ## model selection
-  if(is.null(select)){
-    if(exists("AICc")) aic <- AICc(object,k=k)
-    else aic <- AIC(object,k=k)
-    select <- apply(aic, 2, which.min)
-    select <- sapply(select, 
-      function(s) ifelse(length(s)==0,1,s))
-  }
-  
-  ## grab coef
-  B <- mapply( 
-        function(f,s){
-          s <- min(s,length(f$alpha))
-          c(f$alpha[s],f$beta[,s]) }, 
-        object, select)
- 
-   ## set class and double check correct naming
-  B <- as(as(B[,names(object)],"dgCMatrix"),"dmrcoef")
-  rownames(B) <- c("intercept",rownames(object[[1]]$b))
-  B@lambda <- mapply(
-    function(f,s) 
-      ifelse(is.na(f$lambda[s]),f$lambda[which.min(f$lambda)],f$lambda[s]),
-    object, select)
-  
-  return(B)
-}
-
-logLik.dmr <- function(object, ...){
-  dev <- sapply(object, function(fit) fit$dev)
-  df <- sapply(object, function(fit) fit$df)
-  if(!inherits(dev,"matrix")){
-      nl <- attributes(object)$nlambda
-      dev <- sapply(dev,function(a) c(a,rep(NA,nl-length(a))))
-      df <- sapply(df,function(a) c(a,rep(NA,nl-length(a))))
-    }
-
-  ll <- -0.5*dev
-  attr(ll,"nobs") = attributes(object)$nobs
-  attr(ll,"df") = df
-  class(ll) <- "logLik"
-  ll
+coef.dmr <- function(object, ...){
+  B <- lapply(object,coef, ...)
+  bx <- unlist(lapply(B,function(b) b@x))
+  bi <- unlist(lapply(B,function(b) b@i))
+  bp <- c(0,
+    cumsum(unlist(lapply(B,function(b) b@p[-1]))))
+  Bs <- sparseMatrix(i=bi+1,p=bp,x=bx,
+    dims=c(nrow(B[[1]]),length(B)),
+    dimnames=list(rownames(B[[1]]),names(B)))
+  Bs <- as(as(Bs,"dgCMatrix"),"dmrcoef")
+  return(Bs)
 }
 
 ## method predict functions
 predict.dmr <- function(object, newdata, 
-                    type=c("link","response","class"), ...){
+                  type=c("link","response","class"), ...){
   B <- coef(object, ...)
   predict(B,newdata=newdata,type=type)
 }
@@ -165,10 +106,10 @@ predict.dmrcoef <- function(object, newdata,
   if(is.vector(newdata)){ newdata <- matrix(newdata, nrow=1) }
   if(is.data.frame(newdata)){ newdata <- as.matrix(newdata) }
 
+  type=match.arg(type)
   if(type=="reduction")
     stop("type `reduction' has been replaced by the `srproj' function in the textir library.")
-  type=match.arg(type)
-  
+
   eta <- t(tcrossprod(t(object[-1,,drop=FALSE]),newdata) + object[1,])
   if(type=="response"){
     expeta <- exp(eta)
